//
// Description:
//  components is a container class for all the simulated components, e.g.,
//  client, network, and NASD. When simulating a component, an event first
//  accesses the component using the component class. The event can then
//  gather parameters and possibily use the component to help simulate.
//

// $Id: nasdsim_components.h,v 1.7 2000/04/17 03:02:20 ozaki Exp $


#if !defined(NASDSIM_COMPONENTS_H)
#define NASDSIM_COMPONENTS_H

#include <map>
#include "nasdsim_component_client.h"
#include "nasdsim_component_nasd.h"
#include "nasdsim_types.h"

class component_client;

typedef map <sim_component_id_t, component_client *> clients_list_t;
typedef clients_list_t::const_iterator clients_list_iter_t;

typedef map <sim_component_id_t, component_nasd *> nasds_list_t;
typedef nasds_list_t::const_iterator nasds_list_iter_t;

class components {
 public:
  components(void);
  ~components();

  void add_component_nasd(sim_component_id_t id, component_nasd *comp);
  void add_component_client (sim_component_id_t id, 
			     component_client *comp);

  component_nasd *get_component_nasd(sim_component_id_t id);
  component_client *get_component_client (sim_component_id_t id);
  
  void dump_stats(ostream &output) const;
  
  
  const clients_list_t & getClientList (void) { return clients; }
  const nasds_list_t & getNasdList (void) { return nasds; }
  
 private:
  
  nasds_list_t nasds;     // all of the NASDs in the system indexed by id
  clients_list_t clients; // all clients in system indexed by id
};

#endif // !defined(NASDSIM_COMPONENTS_H)
