// $Id: nasdsim_config.h,v 1.3 2000/04/24 17:25:22 bforney Exp $

#if !defined (NASDSIM_CONFIG_H)
#define NASDSIM_CONFIG_h

#include <list>
#include <fstream.h>
#include "nasdsim_types.h"
#include "nasdsim_component_nasd.h"

#define CONFIG_TRACE_FILENAME_SZ 154

struct networkInfo {
  sim_bandwidth_t bandwidth;
  sim_len_t headerSize;
  simtime_t switchTime;
};

struct nasdInfo {
  sim_component_id_t id;
  disk_bandwidths_t *bandwidths;
  simtime_t seekTime;
  sim_len_t cacheSize;
  sim_len_t blockSize;
  int queueSize;
};

struct clientInfo {
  sim_component_id_t id;
  char filename [CONFIG_TRACE_FILENAME_SZ];
};

class nasdsim_config 
{
 public:
  
  nasdsim_config (char *filename);
  ~nasdsim_config ();
  
  // accessors
  list <nasdInfo *> * getNasdList (void) const { return nasdList; }
  list <clientInfo *> * getClientList (void) const { return clientList; }
  networkInfo * getNetworkInfo (void) const { return nwInfo; }
  
  // static functions (should be able to execute without an instance of this class
  static void cleanUp (list <clientInfo *> * cList);
  static void cleanUp (list <nasdInfo *> * nList);
  
  static clientInfo * getClientInfo (ifstream &fin);
  static nasdInfo *getNasdInfo (ifstream &fin);
  static void * getGeneralInfo (ifstream &fin);
  static networkInfo * getNetworkInfo (ifstream &fin);
  static list <clientInfo *> * getClientList (ifstream &fin);
  static list <nasdInfo *> * getNasdList (ifstream &fin); 
  
  static void print (nasdInfo *);
  static void print (clientInfo *);
  static void print (networkInfo *);
  
 private:
  
  networkInfo *nwInfo;
  list <nasdInfo *> * nasdList;
  list <clientInfo *> *clientList;
  
  void parseFile (ifstream &fin);
};

#endif // !defined (NASDSIM_CONFIG_H)
