//
// Class:
//  tracefile
//
// Description:
//  [see corresponding header file]
//

// $Id: nasdsim_tracefile.C,v 1.12 2000/05/07 03:08:52 ozaki Exp $

#include <fstream>
#include <iomanip>
#include "nasdsim_tracefile.h"
#include "nasdsim_types.h"
#include "nasdsim_event_READ.h"
#include "nasdsim_event_WRITE.h"

//
// tracefile::tracefile() [constructor]
//
// description:
//  construct a C++ object with information about the NASD object
//
// parameters:
//  filename - pathname for the trace file
//
tracefile::tracefile(
		     char *filename,
		     sim_component_id_t cId
		     )
{
  client_id_ = cId;
  trace_file_ = new ifstream(filename);

  if (!(*trace_file_)) {
    delete trace_file_;
    throw;
  }
  
  read_request_id_ = 0;
  read_events(1);
}

//
// tracefile::~tracefile() [destructor]
//
// description:
//  delete the ifstream object
//
// parameters:
//  none
//
tracefile::~tracefile(void)
{
  delete trace_file_;
}

//
// tracefile::front()
//
// description:
//  peek at the front of the queue
//
// parameters:
//  none
//
// return values:
//  pointer to an event
//
const event *
tracefile::front(void) const
{
  return queue_.front();
}

//
// tracefile::pop()
//
// description:
//  remove the front entry in the trace file queue
//
// parameters:
//  none
//
// return values:
//  none
//
void
tracefile::pop(void)
{
  queue_.pop();

  read_events(1);
}

//
// tracefile::pop()
//
// description:
//  remove the front entry in the trace file queue
//
// parameters:
//  none
//
// return values:
//  none
//
bool
tracefile::empty(void)
{
  return trace_file_->eof();
}

//
// tracefile::read_events()
//
// description:
//  parse the tracefile adding new events to the queue.
//
// parameters:
//  num - maximum number of events to read
//
// return values:
//  none
//
void
tracefile::read_events(
		       int num
		       )
{
  simtime_t time;
  char operation_buf[100];
  sim_len_t length;
  sim_object_t object;
  sim_offset_t offset;
  event *tmp;
  sim_stream_id_t stream_id;
  sim_component_id_t nasd_id;
  
  for (; num > 0 && !trace_file_->eof() && !trace_file_->fail(); num--) 
    {
      *trace_file_ >> time >> setw(100) >> operation_buf >> object;
      *trace_file_ >> offset >> length;
      *trace_file_ >> nasd_id >> stream_id;
      
      if (!trace_file_->fail()) {
	  
	  // Remember that the start_time_ of each event coming off
	  // the trace file will change to current time when
	  // that event is placed in the main event heap
	  
	  if (!strcmp(operation_buf, "read")) 
	      {
		  param_read *p = new param_read (object, offset, length, 
						  client_id_, stream_id, nasd_id,
						  read_request_id_, length);
		  tmp = new event_READ_client_read (time, time, p);
		  read_request_id_ ++;
	      }
	else if (!strcmp(operation_buf, "write")) 
	  {
	      tmp = new event_WRITE_client_write (time, time, client_id_, stream_id,
						  nasd_id, object, offset,
						  length);
	  }
	
	queue_.push(tmp);
      }
    }
  if (trace_file_->fail() && !trace_file_->eof()) {
    cerr << "error: cannot read tracefile" << endl;
  }
}

