//
// Description:
//  The tracefile class contains all of the logic to parse a trace textfile.
//  New trace entries are given by checking the "front" of the trace file.
//  (The front is the next trace event.) This class is modeled after a queue.
//

// $Id: nasdsim_tracefile.h,v 1.5 2000/04/24 23:01:55 ozaki Exp $

#if !defined(NASDSIM_TRACEFILE_H)
#define NASDSIM_TRACEFILE_H

#include <fstream>
#include <queue>
#include "nasdsim_event.h"

class event;

class tracefile {
 public:
  tracefile (char *filepath, sim_component_id_t);
  ~tracefile();
  
  sim_component_id_t get_client_id (void) { return client_id_; }
  const event *front(void) const;

  void pop(void);

  bool empty(void);
 private:
  void read_events(int num);

 private:
  queue<event *> queue_; // a queue in case we want to batch process trace
                         // file requests.
  ifstream *trace_file_; // C++ ifstream for the trace file
  sim_component_id_t client_id_;
  sim_id_t read_request_id_;
};

#endif // !defined(NASDSIM_TRACEFILE_H)
