#include <fstream>
#include <iomanip>
#include <stdlib.h>
#include <time.h>
#include <limits.h>
#include <sys/debug.h>
 
struct fileInfo {
  unsigned int id;
  unsigned int size;
  float cash;
};

// prototypes
void dump_read  (ofstream & fout, float, int objId, int offset, int size, 
		 int nasdId, long streamId);

void genRequest (ofstream & fout, 
		 fileInfo &file, 
		 int numNasds,
		 float &,
		 int stripeSize);
void
distributeSize (fileInfo *files, int numFiles);

long stream_id;
float reqGap;
int 
main(int argc, char **argv)
{
  if (argc != 2) {
    cerr << "Usage: genWeb <trace file name>" << endl;
    exit (-1);
  }
  
  ofstream fout (argv [1]);
  if (!fout) { exit (-1); }
 
  
  //
  // Initialize synthetic generator's paramters
  //
  
  unsigned int numNasds    = 2;
  unsigned int stripeSize  = 10240;
  unsigned int numFileReq  = 500;
  float        startTime   = 0;
  unsigned int numFiles    = 100;
  reqGap                   = 0.001;
  
  // End of parameters for the generator -------------------------------------------
  
  
  // Do currency distribution ------------------------------------------------------
  
  // create array of fileInfos
  fileInfo *files = new fileInfo [numFiles];
  
  for (int i = 0; i < numFiles; i++) files [i].id = i;
  
  // distribute cash  = 100 amongst numFiles files
  float availCash = 100;
  float currCash  = 40;
  
  {
    int i;
    
    for (i = 0; i < numFiles - 1; i++) {
      availCash -= currCash;
      files [i].cash = currCash;
      currCash = currCash / (float)2;
    }
    
    // Give numFilesth file the remaining cash
    files [i].cash = availCash;
  } 
  // End of currency distribution---------------------------------------------------
  
  // random number generator
  srand48 (time (0));
  
  // distribute size
  distributeSize (files, numFiles);
  
  // each request for a file will be an independent request. However, the
  // sub-requests for the objects of the file (since the file is striped) will be
  // dependent on each other. Each request for a file will be allocated a
  // monotonically increasing stream_id that all its sub-requests will use
  stream_id = 0;
  
  float currTime           = startTime;
  unsigned long long uniqueBytes = 0;
  
  // number of files is numFiles. So make an array of boolean that will keep track
  // whether this file was referenced before or not.
  bool * fileRef  =  new bool [numFiles];
  for (int i = 0; i < numFiles; i++) fileRef [i] = false;
  
  // Main request generation loop
  for (int i = 0; i < numFileReq; i++) 
    {
      float val = (float)(drand48 () * 100); // Get a random float in [0, 100]
      
      int fileId; // specifies id of selected file
      
      float sum = 0;
      
      {
	int j;
	for (j = 0; j < numFiles; j++) 
	  {
	    sum += files [j].cash;
	    
	    if (sum > val) {
	      fileId = j;
	      break;
	    }
	  }
	
	if (j == numFiles) fileId = numFiles - 1;
      }
      
      if (!fileRef [fileId]) { 
	fileRef [fileId] = true; 
	uniqueBytes += files [fileId].size;
      }
      
      genRequest (fout, files [fileId], numNasds, currTime, stripeSize);
    }
  
  delete files;
  delete fileRef;
  
  cout    << "Unique Bytes touched = " << uniqueBytes << endl;
}

void 
genRequest (ofstream & fout, 
	    fileInfo &file, 
	    int numNasds,
	    float & currTime,
	    int stripeSize) 
{
  int stOid   = file.id * 100;
  int endOid  = stOid + 99;
  int currOid = stOid;
  
  // A simple hack to make striping occur starting at different nasds for files with
  // different ids. The idea is that the workload is better spaced out over the
  // NASDs
    
  int  size   = file.size;
  bool start  = true;
  int  i;
  
  while (size > 0) {
      
    if (start) { 
      start = false; i = file.id % numNasds; 
    }
    else i = 0;
      
    for (; (i < numNasds) && (size > 0); i++) 
      {
	ASSERT (currOid < endOid + 1);
	
	int s = (size > stripeSize)? stripeSize : size;
	
	dump_read (fout, currTime, currOid, 0, s, i, stream_id); fout << endl;
	currTime += reqGap;
	
	size = size - stripeSize;
      }
    
    currOid++;
  }
  
  stream_id++;
  
  // wrap around
  if (stream_id == LONG_MAX) stream_id = 0;
}

void 
dump_read (ofstream & fout, float currTime, int objId, int offset, int size, 
	   int nasdId, long streamId)
{
  fout << currTime << " read " << objId << " " << offset << " " << size << " "
       << nasdId << " " << streamId;
}

/**
 * Do size distribution based crudely on the normal distribution
 */
void
distributeSize (fileInfo *files, int numFiles)
{
  ASSERT (files != NULL && numFiles > 0);
  
  // (cash, file size)
  int cells [][2] = 
  { 
    {5,  3072},
    {10, 6144},
    {66, 12288},
    {10, 102400},
    {5,  204800}
  };
  
  int numCells = 5;
  
  // Maintaining stats on each region
  int stats [5];
  for (int i = 0; i < 5; i++) stats [i] = 0;
  
  for (int i = 0; i < numFiles; i++) 
    {
      float val = (float)(drand48 () * 100); // Get a random float in [0, 100]
      
      int cellIndex; // specifies index of selected cell
      
      float sum = 0;
      
      int j;
      
      for (j = 0; j < numCells; j++) 
	{
	  sum += cells [j][0];
	  if (sum > val) {
	    cellIndex = j;
	    break;
	  }
	}
      
      if (j == numCells) cellIndex = numCells - 1;
      
      stats [cellIndex]++;
      files [i].size = cells [cellIndex][1];
    }

  cout << "STATS for length distribution ***" << endl;
  
  for (int i = 0; i < 5; i++) {
    cout << "Region " << i << ", cnt = " << stats [i] 
	 << ", % of total = " << (stats [i] * 100) / (float)numFiles 
	 << ", size in bytes = " << cells [i][1] << endl;
  }
}
